@@ -20,16 +20,10 @@ public:
                == EventRelation::AnnotationType;
     }
 
-    [[deprecated("Use a two-argument constructor instead")]] // REMOVEME: 0.8
-    explicit ReactionEvent(const EventRelation& er)
-        : EventTemplate(er)
-    {}
     ReactionEvent(const QString& eventId, const QString& reactionKey)
         : EventTemplate(EventRelation::annotate(eventId, reactionKey))
     {}
 
-    [[deprecated("Use eventId(), key(), or content().value instead")]]
-    EventRelation relation() const { return content().value; }
     QString eventId() const { return content().value.eventId; }
     QString key() const { return content().value.key; }
 
